@@ -1,8 +1,35 @@
+use clap::ArgEnum;
 use serde::{Deserialize, Serialize};
 use std::fmt;
 use std::str::FromStr;
 use strum_macros::{EnumDiscriminants, EnumIter, EnumString, ToString};
 
+pub type ClientId = u16; // TODO: merge with crate type?
+
+pub fn client_id_to_colors(
+    client_id: ClientId,
+    colors: Palette,
+) -> Option<(PaletteColor, PaletteColor)> {
+    // (primary color, secondary color)
+    match client_id {
+        1 => Some((colors.magenta, colors.black)),
+        2 => Some((colors.blue, colors.black)),
+        3 => Some((colors.purple, colors.black)),
+        4 => Some((colors.yellow, colors.black)),
+        5 => Some((colors.cyan, colors.black)),
+        6 => Some((colors.gold, colors.black)),
+        7 => Some((colors.red, colors.black)),
+        8 => Some((colors.silver, colors.black)),
+        9 => Some((colors.pink, colors.black)),
+        10 => Some((colors.brown, colors.black)),
+        _ => None,
+    }
+}
+
+pub fn single_client_color(colors: Palette) -> (PaletteColor, PaletteColor) {
+    (colors.green, colors.black)
+}
+
 #[derive(Debug, Copy, Clone, PartialEq, Eq, Hash, Serialize, Deserialize)]
 pub enum Key {
     Backspace,
@@ -31,12 +58,12 @@ pub enum Key {
 // left click) and the `ScrollUp` and `ScrollDown` events could probably be
 // merged into a single `Scroll(isize)` event.
 pub enum Mouse {
-    ScrollUp(usize),                 // number of lines
-    ScrollDown(usize),               // number of lines
-    LeftClick(isize, usize),         // line and column
-    RightClick(isize, usize),        // line and column
-    Hold(isize, usize),              // line and column
-    Release(Option<(isize, usize)>), // line and column
+    ScrollUp(usize),          // number of lines
+    ScrollDown(usize),        // number of lines
+    LeftClick(isize, usize),  // line and column
+    RightClick(isize, usize), // line and column
+    Hold(isize, usize),       // line and column
+    Release(isize, usize),    // line and column
 }
 
 #[derive(Debug, Clone, PartialEq, EnumDiscriminants, ToString, Serialize, Deserialize)]
@@ -49,13 +76,14 @@ pub enum Event {
     Key(Key),
     Mouse(Mouse),
     Timer(f64),
-    CopyToClipboard,
+    CopyToClipboard(CopyDestination),
+    SystemClipboardFailure,
     InputReceived,
     Visible(bool),
 }
 
 /// Describes the different input modes, which change the way that keystrokes will be interpreted.
-#[derive(Debug, PartialEq, Eq, Hash, Copy, Clone, EnumIter, Serialize, Deserialize)]
+#[derive(Debug, PartialEq, Eq, Hash, Copy, Clone, EnumIter, Serialize, Deserialize, ArgEnum)]
 pub enum InputMode {
     /// In `Normal` mode, input is always written to the terminal, except for the shortcuts leading
     /// to other modes
@@ -92,6 +120,9 @@ pub enum InputMode {
     /// `Prompt` mode allows interacting with active prompts.
     #[serde(alias = "prompt")]
     Prompt,
+    /// `Tmux` mode allows for basic tmux keybindings functionality
+    #[serde(alias = "tmux")]
+    Tmux,
 }
 
 impl Default for InputMode {
@@ -136,6 +167,7 @@ impl FromStr for InputMode {
             "renametab" => Ok(InputMode::RenameTab),
             "session" => Ok(InputMode::Session),
             "move" => Ok(InputMode::Move),
+            "tmux" => Ok(InputMode::Tmux),
             "prompt" => Ok(InputMode::Prompt),
             "renamepane" => Ok(InputMode::RenamePane),
             e => Err(e.to_string().into()),
@@ -169,6 +201,11 @@ pub struct Palette {
     pub white: PaletteColor,
     pub orange: PaletteColor,
     pub gray: PaletteColor,
+    pub purple: PaletteColor,
+    pub gold: PaletteColor,
+    pub silver: PaletteColor,
+    pub pink: PaletteColor,
+    pub brown: PaletteColor,
 }
 
 /// Represents the contents of the help message that is printed in the status bar,
@@ -193,6 +230,8 @@ pub struct TabInfo {
     pub panes_to_hide: usize,
     pub is_fullscreen_active: bool,
     pub is_sync_panes_active: bool,
+    pub are_floating_panes_visible: bool,
+    pub other_focused_clients: Vec<ClientId>,
 }
 
 #[derive(Debug, Copy, Clone, PartialEq, Eq, Hash, Deserialize, Serialize)]
@@ -233,3 +272,10 @@ impl Default for PluginCapabilities {
         PluginCapabilities { arrow_fonts: true }
     }
 }
+
+#[derive(Debug, Copy, Clone, PartialEq, Serialize, Deserialize)]
+pub enum CopyDestination {
+    Command,
+    Primary,
+    System,
+}
